@@ -1,3 +1,8 @@
+// $Id: CglClique.cpp 1033 2011-06-19 16:49:13Z stefan $
+// Copyright (C) 2000, International Business Machines
+// Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 #include <cstdio>
 #include <cassert>
 
@@ -22,7 +27,7 @@ CglClique::CglClique(bool setPacking, bool justOriginalRows) :
    sp_row_start(0),
    sp_row_ind(0),
    node_node(0),
-   petol(-1),
+   petol(-1.0),
    do_row_clique(true),
    do_star_clique(true),
    scl_next_node_rule(SCL_MAX_XJ_MAX_DEG),
@@ -78,13 +83,14 @@ CglClique::generateCuts(const OsiSolverInterface& si, OsiCuts & cs,
 			const CglTreeInfo info) const
 {
    int i;
-   bool has_petol_set = petol >= 0;
+   bool has_petol_set = petol != -1.0;
 
    if (! has_petol_set)
       si.getDblParam(OsiPrimalTolerance, petol);
    int numberOriginalRows = si.getNumRows();
    if (info.inTree&&justOriginalRows_)
      numberOriginalRows = info.formulation_rows;
+   int numberRowCutsBefore = cs.sizeRowCuts();
    // First select which rows/columns we are interested in.
    if (!setPacking_) {
       selectFractionalBinaries(si);
@@ -116,6 +122,11 @@ CglClique::generateCuts(const OsiSolverInterface& si, OsiCuts & cs,
       find_rcl(cs);
    if (do_star_clique)
       find_scl(cs);
+   if (!info.inTree&&((info.options&4)==4||((info.options&8)&&!info.pass))) {
+     int numberRowCutsAfter = cs.sizeRowCuts();
+     for (int i=numberRowCutsBefore;i<numberRowCutsAfter;i++)
+       cs.rowCutPtr(i)->setGloballyValid();
+   }
 
    delete[] cl_indices;     cl_indices = 0;
    delete[] cl_del_indices; cl_del_indices = 0;
@@ -404,7 +415,7 @@ CglClique::find_scl(OsiCuts& cs) const
 
 int
 CglClique::scl_choose_next_node(const int current_nodenum,
-				const int *current_indices,
+				const int * /* current_indices */,
 				const int *current_degrees,
 				const double *current_values) const
 {
@@ -479,14 +490,14 @@ CglClique::scl_delete_node(const int del_ind, int& current_nodenum,
 
    /* delete the entry corresponding to del_ind from current_indices, 
       current_degrees and current_values */
-   memmove((char *)(current_indices + del_ind),
-	   (char *)(current_indices + (del_ind+1)),
+   memmove(reinterpret_cast<char *>(current_indices + del_ind),
+	   reinterpret_cast<char *>(current_indices + (del_ind+1)),
 	   (current_nodenum-del_ind-1) * sizeof(int));
-   memmove((char *)(current_degrees + del_ind),
-	   (char *)(current_degrees + (del_ind+1)),
+   memmove(reinterpret_cast<char *>(current_degrees + del_ind),
+	   reinterpret_cast<char *>(current_degrees + (del_ind+1)),
 	   (current_nodenum-del_ind-1) * sizeof(int));
-   memmove((char *)(current_values + del_ind),
-	   (char *)(current_values + (del_ind+1)),
+   memmove(reinterpret_cast<char *>(current_values + del_ind),
+	   reinterpret_cast<char *>(current_values + (del_ind+1)),
 	   (current_nodenum-del_ind-1) * sizeof(double));
    current_nodenum--;
    
@@ -691,12 +702,13 @@ CglClique::recordClique(const int len, int* indices, OsiCuts& cs) const
    for (int j = len - 1; j >= 0; j--)
       indices[j] = sp_orig_col_ind[indices[j]];
    std::sort(indices, indices + len);
-   OsiRowCut* rowcut = new OsiRowCut();
+   OsiRowCut rowcut;
    double* coef = new double[len];
    std::fill(coef, coef + len, 1.0);
-   rowcut->setRow(len, indices, coef);
-   rowcut->setUb(1.0);
-   cs.insert(rowcut);
+   rowcut.setRow(len, indices, coef);
+   rowcut.setUb(1.0);
+   CoinAbsFltEq equal(1.0e-12);
+   cs.insertIfNotDuplicate(rowcut,equal);
    delete[] coef;
 }
 
