@@ -1,5 +1,7 @@
+/* $Id: CoinFactorization.hpp 1448 2011-06-19 15:34:41Z stefan $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 /* 
    Authors
@@ -9,13 +11,16 @@
  */
 #ifndef CoinFactorization_H
 #define CoinFactorization_H
+//#define COIN_ONE_ETA_COPY 100
 
 #include <iostream>
 #include <string>
 #include <cassert>
 #include <cstdio>
-#include "CoinFinite.hpp"
+#include <cmath>
+#include "CoinTypes.hpp"
 #include "CoinIndexedVector.hpp"
+
 class CoinPackedMatrix;
 /** This deals with Factorization and Updates
 
@@ -42,7 +47,6 @@ class CoinPackedMatrix;
  */
 
 
-
 class CoinFactorization {
    friend void CoinFactorizationUnitTest( const std::string & mpsDir );
 
@@ -113,7 +117,7 @@ public:
 		       CoinBigIndex estimateNumberElements,
 		       int * indicesRow[],
 		       int * indicesColumn[],
-		       double * elements[],
+		       CoinFactorizationDouble * elements[],
 		  double areaFactor = 0.0);
   /** This is part two of factorization
       Arrays belong to factorization and were returned by part 1
@@ -152,15 +156,17 @@ public:
     return pivotColumn_.array();
   }
   /// Returns address of pivot region
-  inline double *pivotRegion (  ) const {
+  inline CoinFactorizationDouble *pivotRegion (  ) const {
     return pivotRegion_.array();
   }
   /// Returns address of permuteBack region
   inline int *permuteBack (  ) const {
     return permuteBack_.array();
   }
-  /// Returns address of pivotColumnBack region (also used for permuting)
+  /** Returns address of pivotColumnBack region (also used for permuting)
+      Now uses firstCount to save memory allocation */
   inline int *pivotColumnBack (  ) const {
+    //return firstCount_.array();
     return pivotColumnBack_.array();
   }
   /// Start of each row in L
@@ -180,7 +186,7 @@ public:
   { return indexRowL_.array();}
 
   /// Elements in L (row copy)
-  inline double * elementByRowL() const
+  inline CoinFactorizationDouble * elementByRowL() const
   { return elementByRowL_.array();}
 
   /// Number of Rows after iterating
@@ -325,7 +331,7 @@ public:
   inline int * numberInColumn() const
   { return numberInColumn_.array();}
   /// Elements of U
-  inline double * elementU() const
+  inline CoinFactorizationDouble * elementU() const
   { return elementU_.array();}
   /// Row indices of U
   inline int * indexRowU() const
@@ -366,7 +372,15 @@ public:
   int replaceColumn ( CoinIndexedVector * regionSparse,
 		      int pivotRow,
 		      double pivotCheck ,
-		      bool checkBeforeModifying=false);
+		      bool checkBeforeModifying=false,
+		      double acceptablePivot=1.0e-8);
+  /** Combines BtranU and delete elements
+      If deleted is NULL then delete elements
+      otherwise store where elements are
+  */
+  void replaceColumnU ( CoinIndexedVector * regionSparse,
+			CoinBigIndex * deleted,
+			int internalPivotRow);
   //@}
 
   /**@name various uses of factorization (return code number elements) 
@@ -515,6 +529,9 @@ protected:
   bool getColumnSpace ( int iColumn,
 			int extraNeeded );
 
+  /** Reorders U so contiguous and in order (if there is space)
+      Returns true if it could */
+  bool reorderU();
   /**  getColumnSpaceIterateR.  Gets space for one extra R element in Column
        may have to do compression  (returns true)
        also moves existing vector */
@@ -639,6 +656,10 @@ protected:
   /** Updates part of column transpose (BTRANU) when sparse,
       assumes index is sorted i.e. region is correct */
   void updateColumnTransposeUSparse ( CoinIndexedVector * region) const;
+  /** Updates part of column transpose (BTRANU) by column
+      assumes index is sorted i.e. region is correct */
+  void updateColumnTransposeUByColumn ( CoinIndexedVector * region,
+					int smallestIndex) const;
 
   /// Updates part of column transpose (BTRANR)
   void updateColumnTransposeR ( CoinIndexedVector * region ) const;
@@ -683,9 +704,8 @@ protected:
 	  int pivotColumn,
 	  CoinBigIndex pivotRowPosition,
 	  CoinBigIndex pivotColumnPosition,
-	  double work[],
+	  CoinFactorizationDouble work[],
 	  unsigned int workArea2[],
-	  int increment,
 	  int increment2,
 	  T markRow[] ,
 	  int largeInteger)
@@ -693,11 +713,11 @@ protected:
   int *indexColumnU = indexColumnU_.array();
   CoinBigIndex *startColumnU = startColumnU_.array();
   int *numberInColumn = numberInColumn_.array();
-  double *elementU = elementU_.array();
+  CoinFactorizationDouble *elementU = elementU_.array();
   int *indexRowU = indexRowU_.array();
   CoinBigIndex *startRowU = startRowU_.array();
   int *numberInRow = numberInRow_.array();
-  double *elementL = elementL_.array();
+  CoinFactorizationDouble *elementL = elementL_.array();
   int *indexRowL = indexRowL_.array();
   int *saveColumn = saveColumn_.array();
   int *nextRow = nextRow_.array();
@@ -746,13 +766,13 @@ protected:
 
   if ( l + numberInPivotColumn > lengthAreaL_ ) {
     //need more memory
-    printf("more memory needed in middle of invert\n");
+    if ((messageLevel_&4)!=0) 
+      printf("more memory needed in middle of invert\n");
     return false;
   }
   //l+=currentAreaL_->elementByColumn-elementL;
   CoinBigIndex lSave = l;
 
-  pivotRowL_.array()[numberGoodL_] = pivotRow;
   CoinBigIndex * startColumnL = startColumnL_.array();
   startColumnL[numberGoodL_] = l;	//for luck and first time
   numberGoodL_++;
@@ -815,8 +835,8 @@ protected:
   }
   assert (pivotRowPosition<endColumn);
   assert (indexRowU[pivotRowPosition]==pivotRow);
-  double pivotElement = elementU[pivotRowPosition];
-  double pivotMultiplier = 1.0 / pivotElement;
+  CoinFactorizationDouble pivotElement = elementU[pivotRowPosition];
+  CoinFactorizationDouble pivotMultiplier = 1.0 / pivotElement;
 
   pivotRegion_.array()[numberGoodU_] = pivotMultiplier;
   pivotRowPosition++;
@@ -849,7 +869,7 @@ protected:
   numberInColumn[pivotColumn] = 0;
   //use end of L for temporary space
   int *indexL = &indexRowL[lSave];
-  double *multipliersL = &elementL[lSave];
+  CoinFactorizationDouble *multipliersL = &elementL[lSave];
 
   //adjust
   int j;
@@ -874,11 +894,11 @@ protected:
     CoinBigIndex startColumn = startColumnU[iColumn];
     CoinBigIndex endColumn = startColumn + numberInColumn[iColumn];
     int iRow = indexRowU[startColumn];
-    double value = elementU[startColumn];
+    CoinFactorizationDouble value = elementU[startColumn];
     double largest;
     CoinBigIndex put = startColumn;
     CoinBigIndex positionLargest = -1;
-    double thisPivotValue = 0.0;
+    CoinFactorizationDouble thisPivotValue = 0.0;
 
     //compress column and find largest not updated
     bool checkLargest;
@@ -976,6 +996,7 @@ protected:
 
       if ( absValue > tolerance ) {
 	work[j] = 0.0;
+	assert (put<lengthAreaU_); 
 	elementU[put] = value;
 	indexRowU[put] = indexL[j];
 	if ( absValue > largest ) {
@@ -1274,7 +1295,7 @@ protected:
   CoinIntArrayWithLength pivotRowL_;
 
   /// Inverses of pivot values
-  CoinDoubleArrayWithLength pivotRegion_;
+  CoinFactorizationDoubleArrayWithLength pivotRegion_;
 
   /// Number of slacks at beginning of U
   int numberSlacks_;
@@ -1295,7 +1316,7 @@ protected:
   CoinBigIndex lengthAreaU_;
 
 /// Elements of U
-  CoinDoubleArrayWithLength elementU_;
+  CoinFactorizationDoubleArrayWithLength elementU_;
 
 /// Row indices of U
   CoinIntArrayWithLength indexRowU_;
@@ -1319,7 +1340,7 @@ protected:
   CoinBigIndex lengthAreaL_;
 
   /// Elements of L
-  CoinDoubleArrayWithLength elementL_;
+  CoinFactorizationDoubleArrayWithLength elementL_;
 
   /// Row indices of L
   CoinIntArrayWithLength indexRowL_;
@@ -1340,7 +1361,7 @@ protected:
   CoinBigIndex lengthAreaR_;
 
   /// Elements of R
-  double *elementR_;
+  CoinFactorizationDouble *elementR_;
 
   /// Row indices for R
   int *indexRowR_;
@@ -1361,7 +1382,7 @@ protected:
   int denseThreshold_;
 
   /// First work area
-  CoinDoubleArrayWithLength workArea_;
+  CoinFactorizationDoubleArrayWithLength workArea_;
 
   /// Second work area
   CoinUnsignedIntArrayWithLength workArea2_;
@@ -1407,7 +1428,7 @@ protected:
   CoinIntArrayWithLength indexColumnL_;
 
   /// Elements in L (row copy)
-  CoinDoubleArrayWithLength elementByRowL_;
+  CoinFactorizationDoubleArrayWithLength elementByRowL_;
 
   /// Sparse regions
   mutable CoinIntArrayWithLength sparse_;
@@ -1480,13 +1501,13 @@ typedef const int cipfint;
   {
     if ( l + numberDoColumn > lengthAreaL_ ) {
       //need more memory
-      printf("more memory needed in middle of invert\n");
+      if ((messageLevel_&4)!=0) 
+	printf("more memory needed in middle of invert\n");
       goto BAD_PIVOT;
     }
     //l+=currentAreaL_->elementByColumn-elementL;
     CoinBigIndex lSave = l;
     
-    pivotRowL_.array()[numberGoodL_] = iPivotRow;
     CoinBigIndex * startColumnL = startColumnL_.array();
     startColumnL[numberGoodL_] = l;	//for luck and first time
     numberGoodL_++;
@@ -1549,8 +1570,8 @@ typedef const int cipfint;
     }
     assert (pivotRowPosition<endColumn);
     assert (indexRow[pivotRowPosition]==iPivotRow);
-    double pivotElement = element[pivotRowPosition];
-    double pivotMultiplier = 1.0 / pivotElement;
+    CoinFactorizationDouble pivotElement = element[pivotRowPosition];
+    CoinFactorizationDouble pivotMultiplier = 1.0 / pivotElement;
     
     pivotRegion_.array()[numberGoodU_] = pivotMultiplier;
     pivotRowPosition++;
@@ -1583,7 +1604,7 @@ typedef const int cipfint;
     numberInColumn[iPivotColumn] = 0;
     //use end of L for temporary space
     int *indexL = &indexRowL[lSave];
-    double *multipliersL = &elementL[lSave];
+    CoinFactorizationDouble *multipliersL = &elementL[lSave];
     
     //adjust
     int j;
@@ -1608,11 +1629,11 @@ typedef const int cipfint;
       CoinBigIndex startColumnThis = startColumn[iColumn];
       CoinBigIndex endColumn = startColumnThis + numberInColumn[iColumn];
       int iRow = indexRow[startColumnThis];
-      double value = element[startColumnThis];
+      CoinFactorizationDouble value = element[startColumnThis];
       double largest;
       CoinBigIndex put = startColumnThis;
       CoinBigIndex positionLargest = -1;
-      double thisPivotValue = 0.0;
+      CoinFactorizationDouble thisPivotValue = 0.0;
       
       //compress column and find largest not updated
       bool checkLargest;
