@@ -1,4 +1,4 @@
-/* $Id: CoinFactorization4.cpp 1373 2011-01-03 23:57:44Z lou $ */
+/* $Id: CoinFactorization4.cpp 1552 2012-10-30 17:12:05Z forrest $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
 // This code is licensed under the terms of the Eclipse Public License (EPL).
@@ -1437,10 +1437,18 @@ CoinFactorization::updateColumnTransposeL ( CoinIndexedVector * regionSparse ) c
       if (goSparse>0||!numberL_)
 	regionSparse->scan(lastSparse,numberRows_,zeroTolerance_);
     } 
-    if (!numberL_)
+    if (!numberL_) {
+      // could be odd combination of sparse/dense
+      if (number>numberRows_) {
+	regionSparse->setNumElements(0);
+	regionSparse->scan(0,numberRows_,zeroTolerance_);
+      }
       return;
+    }
   } 
 #endif
+  if (goSparse>0&&regionSparse->getNumElements()>numberRows_)
+    goSparse=0;
   switch (goSparse) {
   case -1: // No row copy
     updateColumnTransposeLDensish(regionSparse);
