@@ -1,19 +1,36 @@
+/* $Id: CoinHelperFunctions.hpp 1448 2011-06-19 15:34:41Z stefan $ */
 // Copyright (C) 2000, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 #ifndef CoinHelperFunctions_H
 #define CoinHelperFunctions_H
+
+#include "CoinUtilsConfig.h"
+
 #if defined(_MSC_VER)
 #  include <direct.h>
 #  define getcwd _getcwd
 #else
 #  include <unistd.h>
 #endif
+//#define USE_MEMCPY
 
 #include <cstdlib>
 #include <cstdio>
+#include <algorithm>
+#include "CoinTypes.hpp"
 #include "CoinError.hpp"
-#include "CoinFinite.hpp"
+
+// Compilers can produce better code if they know about __restrict
+#ifndef COIN_RESTRICT
+#ifdef COIN_USE_RESTRICT
+#define COIN_RESTRICT __restrict
+#else
+#define COIN_RESTRICT
+#endif
+#endif
+
 //#############################################################################
 
 /** This helper function copies an array to another location using Duff's
@@ -27,9 +44,11 @@ CoinCopyN(register const T* from, const int size, register T* to)
     if (size == 0 || from == to)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to copy negative number of entries",
 			"CoinCopyN", "");
+#endif
 
     register int n = (size + 7) / 8;
     if (to > from) {
@@ -70,11 +89,17 @@ CoinCopyN(register const T* from, const int size, register T* to)
 /** This helper function copies an array to another location using Duff's
     device (for a speedup of ~2). The source array is given by its first and
     "after last" entry; the target array is given by its first entry.
-    Overlapping arrays are handled correctly. */
+    Overlapping arrays are handled correctly.
+
+    All of the various CoinCopyN variants use an int for size. On 64-bit
+    architectures, the address diff last-first will be a 64-bit quantity.
+    Given that everything else uses an int, I'm going to choose to kick
+    the difference down to int.  -- lh, 100823 --
+*/
 template <class T> inline void
 CoinCopy(register const T* first, register const T* last, register T* to)
 {
-    CoinCopyN(first, last - first, to);
+    CoinCopyN(first, static_cast<int>(last-first), to);
 }
 
 //-----------------------------------------------------------------------------
@@ -93,9 +118,11 @@ CoinDisjointCopyN(register const T* from, const int size, register T* to)
     if (size == 0 || from == to)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to copy negative number of entries",
 			"CoinDisjointCopyN", "");
+#endif
 
 #if 0
     /* There is no point to do this test. If to and from are from different
@@ -252,9 +279,11 @@ CoinMemcpyN(register const T* from, const int size, register T* to)
     if (size == 0 || from == to)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to copy negative number of entries",
 			"CoinMemcpyN", "");
+#endif
 
 #if 0
     /* There is no point to do this test. If to and from are from different
@@ -294,10 +323,14 @@ CoinMemcpyN(register const T* from, const int size, register T* to)
 template <class T> inline void
 CoinMemcpyN(const T * COIN_RESTRICT from, int size, T* COIN_RESTRICT to)
 {
-  T * put = (T *) to;
-  const T * get = (const T *) from;
+#ifdef USE_MEMCPY
+  std::memcpy(to,from,size*sizeof(T));
+#else
+  T * COIN_RESTRICT put =  to;
+  const T * COIN_RESTRICT get = from;
   for ( ; 0<size ; --size)
     *put++ = *get++;
+#endif
 }
 #endif
 
@@ -328,10 +361,11 @@ CoinFillN(register T* to, const int size, register const T value)
     if (size == 0)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to fill negative number of entries",
 			"CoinFillN", "");
-
+#endif
 #if 1
     for (register int n = size / 8; n > 0; --n, to += 8) {
 	to[0] = value;
@@ -406,9 +440,11 @@ CoinZeroN(register T* to, const int size)
     if (size == 0)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("trying to fill negative number of entries",
 			"CoinZeroN", "");
+#endif
 #if 1
     for (register int n = size / 8; n > 0; --n, to += 8) {
 	to[0] = 0;
@@ -494,7 +530,7 @@ inline char * CoinStrdup(const char * name)
   char* dup = NULL;
   if (name) {
     const int len = static_cast<int>(strlen(name));
-    dup = (char*)malloc(len+1);
+    dup = static_cast<char*>(malloc(len+1));
     CoinMemcpyN(name, len, dup);
     dup[len] = 0;
   }
@@ -545,9 +581,10 @@ CoinIsSorted(register const T* first, const int size)
     if (size == 0)
 	return true;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("negative number of entries", "CoinIsSorted", "");
-
+#endif
 #if 1
     // size1 is the number of comparisons to be made
     const int size1 = size  - 1;
@@ -604,9 +641,10 @@ CoinIotaN(register T* first, const int size, register T init)
     if (size == 0)
 	return;
 
+#ifndef NDEBUG
     if (size < 0)
 	throw CoinError("negative number of entries", "CoinIotaN", "");
-
+#endif
 #if 1
     for (register int n = size / 8; n > 0; --n, first += 8, init += 8) {
 	first[0] = init;
@@ -669,7 +707,7 @@ template <class T> inline T *
 CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 			   const int * firstDelPos, const int * lastDelPos)
 {
-    int delNum = lastDelPos - firstDelPos;
+    int delNum = static_cast<int>(lastDelPos - firstDelPos);
     if (delNum == 0)
 	return arrayLast;
 
@@ -684,7 +722,8 @@ CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 	delSortedPos = new int[delNum];
 	CoinDisjointCopy(firstDelPos, lastDelPos, delSortedPos);
 	std::sort(delSortedPos, delSortedPos + delNum);
-	delNum = std::unique(delSortedPos, delSortedPos + delNum) - delSortedPos;
+	delNum = static_cast<int>(std::unique(delSortedPos,
+				  delSortedPos+delNum) - delSortedPos);
     }
     const int * delSorted = delSortedPos ? delSortedPos : firstDelPos;
 
@@ -698,7 +737,7 @@ CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 	size += copyLast - copyFirst;
     }
     const int copyFirst = delSorted[last] + 1;
-    const int copyLast = arrayLast - arrayFirst;
+    const int copyLast = static_cast<int>(arrayLast - arrayFirst);
     CoinCopy(arrayFirst + copyFirst, arrayFirst + copyLast,
 	     arrayFirst + size);
     size += copyLast - copyFirst;
@@ -717,23 +756,31 @@ CoinDeleteEntriesFromArray(register T * arrayFirst, register T * arrayLast,
 /* Thanks to Stefano Gliozzi for providing an operating system
    independent random number generator.  */
 
-// linear congruential generator. given the seed, the generated numbers are  
-// always the same regardless the (32 bit) architecture. This allows to 
-// build & test in different environments (i.e. Wintel, Linux/Intel AIX Power5)
-// getting in most cases the same optimization path. 
-/// Return random number between 0 and 1.
-inline double CoinDrand48(bool isSeed = false, unsigned int seed=1)
+/*! \brief Return a random number between 0 and 1
+
+  A platform-independent linear congruential generator. For a given seed, the
+  generated sequence is always the same regardless of the (32-bit)
+  architecture. This allows to build & test in different environments, getting
+  in most cases the same optimization path.
+
+  Set \p isSeed to true and supply an integer seed to set the seed
+  (vid. #CoinSeedRandom)
+
+  \todo Anyone want to volunteer an upgrade for 64-bit architectures?
+*/
+inline double CoinDrand48 (bool isSeed = false, unsigned int seed = 1)
 {
   static unsigned int last = 123456;
   if (isSeed) { 
     last = seed;
   } else {
     last = 1664525*last+1013904223;
-    return (((double) last)/4294967296.0);
+    return ((static_cast<double> (last))/4294967296.0);
   }
-  return(0.0);
+  return (0.0);
 }
-/// Seed random number generator
+
+/// Set the seed for the random number generator
 inline void CoinSeedRandom(int iseed)
 {
   CoinDrand48(true, iseed);
@@ -743,12 +790,16 @@ inline void CoinSeedRandom(int iseed)
 
 #if defined(_MSC_VER) || defined(__MINGW32__) || defined(__CYGWIN32__)
 
+/// Return a random number between 0 and 1
 inline double CoinDrand48() { return rand() / (double) RAND_MAX; }
+/// Set the seed for the random number generator
 inline void CoinSeedRandom(int iseed) { srand(iseed + 69822); }
 
 #else
 
+/// Return a random number between 0 and 1
 inline double CoinDrand48() { return drand48(); }
+/// Set the seed for the random number generator
 inline void CoinSeedRandom(int iseed) { srand48(iseed + 69822); }
 
 #endif
@@ -816,19 +867,22 @@ template <class T> inline void CoinSwap (T &x, T &y)
 */
 
 template <class T> inline int
-CoinToFile( const T* array, int size, FILE * fp)
+CoinToFile( const T* array, CoinBigIndex size, FILE * fp)
 {
-    int numberWritten;
+    CoinBigIndex numberWritten;
     if (array&&size) {
-	numberWritten = fwrite(&size,sizeof(int),1,fp);
+	numberWritten =
+	    static_cast<CoinBigIndex>(fwrite(&size,sizeof(int),1,fp));
 	if (numberWritten!=1)
 	    return 1;
-	numberWritten = fwrite(array,sizeof(T),size,fp);
+	numberWritten =
+	    static_cast<CoinBigIndex>(fwrite(array,sizeof(T),size_t(size),fp));
 	if (numberWritten!=size)
 	    return 1;
     } else {
 	size = 0;
-	numberWritten = fwrite(&size,sizeof(int),1,fp);
+	numberWritten = 
+	    static_cast<CoinBigIndex>(fwrite(&size,sizeof(int),1,fp));
 	if (numberWritten!=1)
 	    return 1;
     }
@@ -844,10 +898,11 @@ CoinToFile( const T* array, int size, FILE * fp)
 */
 
 template <class T> inline int
-CoinFromFile( T* &array, int size, FILE * fp,int & newSize)
+CoinFromFile( T* &array, CoinBigIndex size, FILE * fp, CoinBigIndex & newSize)
 {
-    int numberRead;
-    numberRead = fread(&newSize,sizeof(int),1,fp);
+    CoinBigIndex numberRead;
+    numberRead =
+        static_cast<CoinBigIndex>(fread(&newSize,sizeof(int),1,fp));
     if (numberRead!=1)
 	return 1;
     int returnCode=0;
@@ -855,7 +910,8 @@ CoinFromFile( T* &array, int size, FILE * fp,int & newSize)
 	returnCode=2;
     if (newSize) {
 	array = new T [newSize];
-	numberRead = fread(array,sizeof(T),newSize,fp);
+	numberRead =
+	    static_cast<CoinBigIndex>(fread(array,sizeof(T),newSize,fp));
 	if (numberRead!=newSize)
 	    returnCode=1;
     } else {
@@ -867,6 +923,7 @@ CoinFromFile( T* &array, int size, FILE * fp,int & newSize)
 //#############################################################################
 
 /// Cube Root
+#if 0
 inline double CoinCbrt(double x)
 {
 #if defined(_MSC_VER) 
@@ -875,6 +932,19 @@ inline double CoinCbrt(double x)
     return cbrt(x);
 #endif
 }
+#endif
+
+//-----------------------------------------------------------------------------
+
+/// This helper returns "sizeof" as an int 
+#define CoinSizeofAsInt(type) (static_cast<int>(sizeof(type)))
+/// This helper returns "strlen" as an int 
+inline int
+CoinStrlenAsInt(const char * string)
+{
+    return static_cast<int>(strlen(string));
+}
+
 /** Class for thread specific random numbers
 */
 #if defined COIN_OWN_RANDOM_32
@@ -915,12 +985,17 @@ public:
   { 
     seed_ = seed;
   }
+  /** Get seed. */
+  inline unsigned int getSeed() const
+  { 
+    return seed_;
+  }
   /// return a random number
   inline double randomDouble() const
   {
     double retVal;
     seed_ = 1664525*(seed_)+1013904223;
-    retVal = (((double) seed_)/4294967296.0);
+    retVal = ((static_cast<double> (seed_))/4294967296.0);
     return retVal;
   }
   //@}
@@ -1008,4 +1083,9 @@ protected:
   //@}
 };
 #endif
+#ifndef COIN_DETAIL
+#define COIN_DETAIL_PRINT(s) {}
+#else
+#define COIN_DETAIL_PRINT(s) s
+#endif
 #endif
