@@ -1,4 +1,4 @@
-/* $Id: CoinLpIO.cpp 1448 2011-06-19 15:34:41Z stefan $ */
+/* $Id: CoinLpIO.cpp 1545 2012-07-29 18:35:03Z unxusr $ */
 // Last edit: 11/5/08
 //
 // Name:     CoinLpIO.cpp; Support for Lp files
@@ -2221,8 +2221,6 @@ CoinLpIO::startHash(char const * const * const names,
 void
 CoinLpIO::stopHash(int section)
 {
-  char **names = names_[section];
-
   freePreviousNames(section);
   previous_names_[section] = names_[section];
   card_previous_names_[section] = numberHash_[section];
@@ -2230,8 +2228,6 @@ CoinLpIO::stopHash(int section)
   delete[] hash_[section];
   hash_[section] = NULL;
 
-  names = NULL;
-
   maxHash_[section] = 0;
   numberHash_[section] = 0;
 
