@@ -1,9 +1,12 @@
+/* $Id: CoinPresolveDoubleton.cpp 1448 2011-06-19 15:34:41Z stefan $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 #include <stdio.h>
 #include <math.h>
 
+#include "CoinFinite.hpp"
 #include "CoinHelperFunctions.hpp"
 #include "CoinPresolveMatrix.hpp"
 
@@ -55,7 +58,11 @@ namespace {	/* begin unnamed local namespace */
    and consistency will be restored.
 */
 
-bool elim_doubleton (const char *msg,
+  bool elim_doubleton (const char * 
+#ifdef PRESOLVE_DEBUG
+msg
+#endif
+		       ,
 		     CoinBigIndex *mcstrt, 
 		     double *rlo, double *rup,
 		     double *colels,
@@ -65,7 +72,11 @@ bool elim_doubleton (const char *msg,
 		     CoinBigIndex *mrstrt, double *rowels,
 		     double coeff_factor,
 		     double bounds_factor,
-		     int row0, int icolx, int icoly)
+		       int
+#ifdef PRESOLVE_DEBUG
+		       row0
+#endif
+		       , int icolx, int icoly)
 
 {
   CoinBigIndex kcsx = mcstrt[icolx];
@@ -205,10 +216,10 @@ const CoinPresolveAction
   action * actions = new action [nrows];
   int nactions = 0;
 
-  int *zeros	= new int[ncols];
+  int *zeros	= prob->usefulColumnInt_; //new int[ncols];
   int nzeros	= 0;
 
-  int *fixed	= new int[ncols];
+  int *fixed	= zeros+ncols; //new int[ncols];
   int nfixed	= 0;
 
   unsigned char *rowstat = prob->rowstat_;
@@ -337,7 +348,7 @@ const CoinPresolveAction
 	    if (rhsRatio==floor(rhsRatio+0.5))
 	      canDo=true;
 	  }
-#ifdef COIN_DEVELOP
+#ifdef COIN_DEVELOP2
 	  if (canDo)
 	    printf("Good CoinPresolveDoubleton icolx %d (%g and bounds %g %g) icoly %d (%g and bound %g %g) - rhs %g\n",
 		   icolx,colels[krowx],clo[icolx],cup[icolx],
@@ -419,6 +430,8 @@ const CoinPresolveAction
 
 	// common equations are of the form ax + by = 0, or x + y >= lo
 	{
+	  PRESOLVE_DETAIL_PRINT(printf("pre_doubleton %dC %dC %dR E\n",
+				       icoly,icolx,irow));
 	  action *s = &actions[nactions];	  
 	  nactions++;
 	  
@@ -672,8 +685,8 @@ const CoinPresolveAction
       next = remove_fixed_action::presolve(prob, fixed, nfixed, next);
   }
 
-  delete[]zeros;
-  delete[]fixed;
+  //delete[]zeros;
+  //delete[]fixed;
   deleteAction(actions,action*);
 
   if (prob->tuning_) {
@@ -998,7 +1011,7 @@ void doubleton_action::postsolve(CoinPostsolveMatrix *prob) const
     } else {
       int ncolx=f->ncolx-1;
       double multiplier = -coeffy/coeffx;
-      int * indx = (int *) (f->colel+ncolx);
+      int * indx = reinterpret_cast<int *> (f->colel+ncolx);
       //printf("Current colx %d\n",jcolx);
 /*
   Scan existing column x to find the end. While we're at it, accumulate part
@@ -1237,6 +1250,8 @@ void doubleton_action::postsolve(CoinPostsolveMatrix *prob) const
     cdone[jcoly] = DOUBLETON;
     rdone[irow] = DOUBLETON;
     presolve_check_threads(prob) ;
+#endif
+# if PRESOLVE_DEBUG
 /*
   Confirm accuracy of reduced cost for columns x and y.
 */
@@ -1307,7 +1322,8 @@ void check_doubletons(const CoinPresolveAction * paction)
     check_doubletons(paction->next);
     
     if (strcmp(paction0->name(), "doubleton_action") == 0) {
-      const doubleton_action *daction = (const doubleton_action *)paction0;
+      const doubleton_action *daction = 
+	reinterpret_cast<const doubleton_action *>(paction0);
       for (int i=daction->nactions_-1; i>=0; --i) {
 	int icolx = daction->actions_[i].icolx;
 	int icoly = daction->actions_[i].icoly;
@@ -1321,8 +1337,13 @@ void check_doubletons(const CoinPresolveAction * paction)
   }
 }
 
+#if	PRESOLVE_DEBUG
 void check_doubletons1(const CoinPresolveAction * paction,
 		       int ncols)
+#else
+void check_doubletons1(const CoinPresolveAction * /*paction*/,
+		       int /*ncols*/)
+#endif
 {
 #if	PRESOLVE_DEBUG
   doubleton_mult = new double[ncols];
