@@ -1,5 +1,8 @@
+/* $Id: CoinPresolveDual.cpp 1373 2011-01-03 23:57:44Z lou $ */
 // Copyright (C) 2002, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
+
 #include <stdio.h>
 #include <math.h>
 
@@ -74,6 +77,7 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
   double *rup	= prob->rup_;
 
   double *dcost	= prob->cost_;
+  const unsigned char *integerType = prob->integerType_;
 
   const double maxmin	= prob->maxmin_;
 
@@ -83,8 +87,8 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
 
   CoinRelFltEq relEq(prob->ztolzb_) ;
 
-  double *rdmin	= new double[nrows];
-  double *rdmax	= new double[nrows];
+  double *rdmin	= prob->usefulRowDouble_; //new double[nrows];
+  double *rdmax	= reinterpret_cast<double *> (prob->usefulRowInt_); //new double[nrows];
 
 # if PRESOLVE_DEBUG
   std::cout << "Entering remove_dual_action::presolve, " << nrows << " X " << ncols << "." << std::endl ;
@@ -115,6 +119,8 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
   // Take the min of the maxes and max of the mins
   int j;
   for ( j = 0; j<ncols; j++) {
+    if (integerType[j])
+      continue; // even if it has infinite bound now ....
     bool no_ub = (cup[j] >= ekkinf);
     bool no_lb = (clo[j] <= -ekkinf);
     
@@ -143,23 +149,23 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
 	//		==> dprice is an upper bound on dj if no *lb*
 	if (rdmax[row] > dprice)	// reduced cost may be positive
 	  rdmax[row] = dprice;
-      } else {			// no lower bound
+      } else if ((coeff < 0.0) == no_lb) { // no lower bound
 	if (rdmin[row] < dprice) 	// reduced cost may be negative
 	  rdmin[row] = dprice;
       }
     }
   }
 
-  int *fixup_cols	= new int[ncols];
+  int *fix_cols	= prob->usefulColumnInt_; //new int[ncols];
 
-  int *fixdown_cols	= new int[ncols];
+  //int *fixdown_cols	= new int[ncols];
 
 #if	PRESOLVE_TIGHTEN_DUALS
   double *djmin	= new double[ncols];
   double *djmax	= new double[ncols];
 #endif
   int nfixup_cols	= 0;
-  int nfixdown_cols	= 0;
+  int nfixdown_cols	= ncols;
 
   int nPass=100;
   while (nPass-->0) {
@@ -211,113 +217,137 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
 	  }
 	}
 	// See if we may be able to tighten a dual
-	if (cup[j]>ekkinf) {
-	  // dj can not be negative
-	  if (nflagu==1&&ddjhi<-ztoldj) {
-	    // We can make bound finite one way
-	    for (CoinBigIndex k = kcs; k < kce; k++) {
-	      int i = hrow[k];
-	      double coeff = colels[k];
-	      
-	      if (coeff > 0.0&&rdmin[i] < -ekkinf2) {
-		// rdmax[i] has upper bound
-		if (ddjhi<rdmax[i]*coeff-ztoldj) {
-		  double newValue = ddjhi/coeff;
-		  // re-compute lo
-		  if (rdmax[i] > ekkinf2 && newValue <= ekkinf2) {
-		    nflagl--;
-		    ddjlo -= coeff * newValue;
-		  } else if (rdmax[i] <= ekkinf2) {
-		    ddjlo -= coeff * (newValue-rdmax[i]);
+	if (!integerType[j]) {
+	  if (cup[j]>ekkinf) {
+	    // dj can not be negative
+	    if (nflagu==1&&ddjhi<-ztoldj) {
+	      // We can make bound finite one way
+	      for (CoinBigIndex k = kcs; k < kce; k++) {
+		int i = hrow[k];
+		double coeff = colels[k];
+		
+		if (coeff > 0.0&&rdmin[i] < -ekkinf2) {
+		  // rdmax[i] has upper bound
+		  if (ddjhi<rdmax[i]*coeff-ztoldj) {
+		    double newValue = ddjhi/coeff;
+		    // re-compute lo
+		    if (rdmax[i] > ekkinf2 && newValue <= ekkinf2) {
+		      nflagl--;
+		      ddjlo -= coeff * newValue;
+		    } else if (rdmax[i] <= ekkinf2) {
+		      ddjlo -= coeff * (newValue-rdmax[i]);
+		    }
+		    rdmax[i] = newValue;
+		    tightened++;
+#if	PRESOLVE_DEBUG
+		    printf("Col %d, row %d max pi now %g\n",j,i,rdmax[i]);
+#endif
 		  }
-		  rdmax[i] = newValue;
+		} else if (coeff < 0.0 && rdmax[i] > ekkinf2) {
+		  // rdmin[i] has lower bound
+		  if (ddjhi<rdmin[i]*coeff-ztoldj) {
+		    double newValue = ddjhi/coeff;
+		    // re-compute lo
+		    if (rdmin[i] < -ekkinf2 && newValue >= -ekkinf2) {
+		      nflagl--;
+		      ddjlo -= coeff * newValue;
+		    } else if (rdmax[i] >= -ekkinf2) {
+		      ddjlo -= coeff * (newValue-rdmin[i]);
+		    }
+		    rdmin[i] = newValue;
+		    tightened++;
+#if	PRESOLVE_DEBUG
+		    printf("Col %d, row %d min pi now %g\n",j,i,rdmin[i]);
+#endif
+		    ddjlo = 0.0;
+		  }
+		}
+	      }
+	    } else if (nflagl==0&&nordl==1&&ddjlo<-ztoldj) {
+	      // We may be able to tighten
+	      for (CoinBigIndex k = kcs; k < kce; k++) {
+		int i = hrow[k];
+		double coeff = colels[k];
+		
+		if (coeff > 0.0) {
+		  rdmax[i] += ddjlo/coeff;
+		  ddjlo =0.0;
 		  tightened++;
 #if	PRESOLVE_DEBUG
 		  printf("Col %d, row %d max pi now %g\n",j,i,rdmax[i]);
 #endif
-		}
-	      } else if (coeff < 0.0 && rdmax[i] > ekkinf2) {
-		// rdmin[i] has lower bound
-		if (ddjhi<rdmin[i]*coeff-ztoldj) {
-		  double newValue = ddjhi/coeff;
-		  // re-compute lo
-		  if (rdmin[i] < -ekkinf2 && newValue >= -ekkinf2) {
-		    nflagl--;
-		    ddjlo -= coeff * newValue;
-		  } else if (rdmax[i] >= -ekkinf2) {
-		    ddjlo -= coeff * (newValue-rdmin[i]);
-		  }
-		  rdmin[i] = newValue;
+		} else if (coeff < 0.0 ) {
+		  rdmin[i] += ddjlo/coeff;
+		  ddjlo =0.0;
 		  tightened++;
 #if	PRESOLVE_DEBUG
 		  printf("Col %d, row %d min pi now %g\n",j,i,rdmin[i]);
 #endif
-		  ddjlo = 0.0;
 		}
 	      }
 	    }
-	  } else if (nflagl==0&&nordl==1&&ddjlo<-ztoldj) {
-	    // We may be able to tighten
-	    for (CoinBigIndex k = kcs; k < kce; k++) {
-	      int i = hrow[k];
-	      double coeff = colels[k];
-	      
-	      if (coeff > 0.0) {
-		rdmax[i] += ddjlo/coeff;
-		ddjlo =0.0;
-		tightened++;
-#if	PRESOLVE_DEBUG
-		printf("Col %d, row %d max pi now %g\n",j,i,rdmax[i]);
-#endif
-	      } else if (coeff < 0.0 ) {
-		rdmin[i] += ddjlo/coeff;
-		ddjlo =0.0;
-		tightened++;
-#if	PRESOLVE_DEBUG
-		printf("Col %d, row %d min pi now %g\n",j,i,rdmin[i]);
-#endif
-	      }
-	    }
 	  }
-	}
 #if 0
-	if (clo[j]<-ekkinf) {
-	  // dj can not be positive
-	  if (ddjlo>ztoldj&&nflagl==1) {
-	    // We can make bound finite one way
-	    for (CoinBigIndex k = kcs; k < kce; k++) {
-	      int i = hrow[k];
-	      double coeff = colels[k];
-	      
-	      if (coeff < 0.0&&rdmin[i] < -ekkinf2) {
-		// rdmax[i] has upper bound
-		if (ddjlo>rdmax[i]*coeff+ztoldj) {
-		  double newValue = ddjlo/coeff;
-		  // re-compute hi
-		  if (rdmax[i] > ekkinf2 && newValue <= ekkinf2) {
-		    nflagu--;
-		    ddjhi -= coeff * newValue;
-		  } else if (rdmax[i] <= ekkinf2) {
-		    ddjhi -= coeff * (newValue-rdmax[i]);
+	  if (clo[j]<-ekkinf) {
+	    // dj can not be positive
+	    if (ddjlo>ztoldj&&nflagl==1) {
+	      // We can make bound finite one way
+	      for (CoinBigIndex k = kcs; k < kce; k++) {
+		int i = hrow[k];
+		double coeff = colels[k];
+		
+		if (coeff < 0.0&&rdmin[i] < -ekkinf2) {
+		  // rdmax[i] has upper bound
+		  if (ddjlo>rdmax[i]*coeff+ztoldj) {
+		    double newValue = ddjlo/coeff;
+		    // re-compute hi
+		    if (rdmax[i] > ekkinf2 && newValue <= ekkinf2) {
+		      nflagu--;
+		      ddjhi -= coeff * newValue;
+		    } else if (rdmax[i] <= ekkinf2) {
+		      ddjhi -= coeff * (newValue-rdmax[i]);
+		    }
+		    rdmax[i] = newValue;
+		    tightened++;
+#if	PRESOLVE_DEBUG
+		    printf("Col %d, row %d max pi now %g\n",j,i,rdmax[i]);
+#endif
 		  }
-		  rdmax[i] = newValue;
+		} else if (coeff > 0.0 && rdmax[i] > ekkinf2) {
+		  // rdmin[i] has lower bound
+		  if (ddjlo>rdmin[i]*coeff+ztoldj) {
+		    double newValue = ddjlo/coeff;
+		    // re-compute lo
+		    if (rdmin[i] < -ekkinf2 && newValue >= -ekkinf2) {
+		      nflagu--;
+		      ddjhi -= coeff * newValue;
+		    } else if (rdmax[i] >= -ekkinf2) {
+		      ddjhi -= coeff * (newValue-rdmin[i]);
+		    }
+		    rdmin[i] = newValue;
+		    tightened++;
+#if	PRESOLVE_DEBUG
+		    printf("Col %d, row %d min pi now %g\n",j,i,rdmin[i]);
+#endif
+		  }
+		}
+	      }
+	    } else if (nflagu==0&&nordu==1&&ddjhi>ztoldj) {
+	      // We may be able to tighten
+	      for (CoinBigIndex k = kcs; k < kce; k++) {
+		int i = hrow[k];
+		double coeff = colels[k];
+		
+		if (coeff < 0.0) {
+		  rdmax[i] += ddjhi/coeff;
+		  ddjhi =0.0;
 		  tightened++;
 #if	PRESOLVE_DEBUG
 		  printf("Col %d, row %d max pi now %g\n",j,i,rdmax[i]);
 #endif
-		}
-	      } else if (coeff > 0.0 && rdmax[i] > ekkinf2) {
-		// rdmin[i] has lower bound
-		if (ddjlo>rdmin[i]*coeff+ztoldj) {
-		  double newValue = ddjlo/coeff;
-		  // re-compute lo
-		  if (rdmin[i] < -ekkinf2 && newValue >= -ekkinf2) {
-		    nflagu--;
-		    ddjhi -= coeff * newValue;
-		  } else if (rdmax[i] >= -ekkinf2) {
-		    ddjhi -= coeff * (newValue-rdmin[i]);
-		  }
-		  rdmin[i] = newValue;
+		} else if (coeff > 0.0 ) {
+		  rdmin[i] += ddjhi/coeff;
+		  ddjhi =0.0;
 		  tightened++;
 #if	PRESOLVE_DEBUG
 		  printf("Col %d, row %d min pi now %g\n",j,i,rdmin[i]);
@@ -325,31 +355,9 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
 		}
 	      }
 	    }
-	  } else if (nflagu==0&&nordu==1&&ddjhi>ztoldj) {
-	    // We may be able to tighten
-	    for (CoinBigIndex k = kcs; k < kce; k++) {
-	      int i = hrow[k];
-	      double coeff = colels[k];
-	      
-	      if (coeff < 0.0) {
-		rdmax[i] += ddjhi/coeff;
-		ddjhi =0.0;
-		tightened++;
-#if	PRESOLVE_DEBUG
-		printf("Col %d, row %d max pi now %g\n",j,i,rdmax[i]);
-#endif
-	      } else if (coeff > 0.0 ) {
-		rdmin[i] += ddjhi/coeff;
-		ddjhi =0.0;
-		tightened++;
-#if	PRESOLVE_DEBUG
-		printf("Col %d, row %d min pi now %g\n",j,i,rdmin[i]);
-#endif
-	      }
-	    }
 	  }
-	}
 #endif
+	}
 
 #if	PRESOLVE_TIGHTEN_DUALS
 	djmin[j] = (nflagl ?  -PRESOLVE_INF : ddjlo);
@@ -366,9 +374,10 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
 	    prob->status_ |= 2;
 	    break;
 	  } else {
-	    fixdown_cols[nfixdown_cols++] = j;
+	    fix_cols[--nfixdown_cols] = j;
+	    PRESOLVE_DETAIL_PRINT(printf("pre_duallo %dC E\n",j));
 #	    if PRESOLVE_DEBUG
-	    printf("NDUAL: fixing x<%d>",fixdown_cols[nfixdown_cols-1]) ;
+	    printf("NDUAL: fixing x<%d>",fix_cols[nfixdown_cols]) ;
 	    if (csol) printf(" = %g",csol[j]) ;
 	    printf(" at lb = %g.\n",clo[j]) ;
 #	    endif
@@ -441,9 +450,10 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
 	    prob->status_ |= 2;
 	    break;
 	  } else {
-	    fixup_cols[nfixup_cols++] = j;
+	    PRESOLVE_DETAIL_PRINT(printf("pre_dualup %dC E\n",j));
+	    fix_cols[nfixup_cols++] = j;
 #	    if PRESOLVE_DEBUG
-	    printf("NDUAL: fixing x<%d>",fixup_cols[nfixup_cols-1]) ;
+	    printf("NDUAL: fixing x<%d>",fix_cols[nfixup_cols-1]) ;
 	    if (csol) printf(" = %g",csol[j]) ;
 	    printf(" at ub = %g.\n",cup[j]) ;
 #	    endif
@@ -579,24 +589,26 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
     }
 #endif
 
-    if (tightened<100||nfixdown_cols||nfixup_cols)
+    if (tightened<100||nfixdown_cols<ncols||nfixup_cols)
       break;
 #if	PRESOLVE_TIGHTEN_DUALS
     else
       printf("DUAL TIGHTENED!  %d\n", tightened);
 #endif
   }
-
+  assert (nfixup_cols<=nfixdown_cols);
   if (nfixup_cols) {
 #if	PRESOLVE_DEBUG
     printf("NDUAL: %d up", nfixup_cols);
-    for (i = 0 ; i < nfixup_cols ; i++) printf(" %d",fixup_cols[i]) ;
+    for (i = 0 ; i < nfixup_cols ; i++) printf(" %d",fix_cols[i]) ;
     printf(".\n") ;
 #endif
-    next = make_fixed_action::presolve(prob, fixup_cols, nfixup_cols, false, next);
+    next = make_fixed_action::presolve(prob, fix_cols, nfixup_cols, false, next);
   }
 
-  if (nfixdown_cols) {
+  if (nfixdown_cols<ncols) {
+    int * fixdown_cols = fix_cols+nfixdown_cols; 
+    nfixdown_cols = ncols-nfixdown_cols;
 #if	PRESOLVE_DEBUG
     printf("NDUAL: %d down", nfixdown_cols);
     for (i = 0 ; i < nfixdown_cols ; i++) printf(" %d",fixdown_cols[i]) ;
@@ -613,7 +625,7 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
   of the interference zone for the typical situation where sizeof(double) is
   twice sizeof(int).
 */
-  int * canFix = (int *) rdmin;
+  int * canFix = reinterpret_cast<int *> (rdmin);
   for ( i = 0; i < nrows; i++) {
     bool no_lb = (rlo[i] <= -ekkinf);
     bool no_ub = (rup[i] >= ekkinf);
@@ -633,6 +645,8 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
   for (j = 0; j<ncols; j++) {
     if (hincol[j]<=1)
       continue;
+    if (integerType[j]) 
+      continue; // even if it has infinite bound now ....
     CoinBigIndex kcs = mcstrt[j];
     CoinBigIndex kce = kcs + hincol[j];
     int bindingUp=-1;
@@ -703,7 +717,6 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
   //const int *hcol	= prob->hcol_;
   //const CoinBigIndex *mrstrt	= prob->mrstrt_;
   //int *hinrow	= prob->hinrow_;
-  const unsigned char *integerType = prob->integerType_;
   for ( i = 0; i < nrows; i++) {
     if (abs(canFix[i])==1) {
       CoinBigIndex krs = mrstrt[i];
@@ -728,11 +741,11 @@ const CoinPresolveAction *remove_dual_action::presolve(CoinPresolveMatrix *prob,
     }
   }
 
-  delete[]rdmin;
-  delete[]rdmax;
+  //delete[]rdmin;
+  //delete[]rdmax;
 
-  delete[]fixup_cols;
-  delete[]fixdown_cols;
+  //delete[]fixup_cols;
+  //delete[]fixdown_cols;
 
 #if	PRESOLVE_TIGHTEN_DUALS
   delete[]djmin;
