@@ -1,9 +1,12 @@
+/* $Id: CoinPresolveTripleton.cpp 1448 2011-06-19 15:34:41Z stefan $ */
 // Copyright (C) 2003, International Business Machines
 // Corporation and others.  All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 
 #include <stdio.h>
 #include <math.h>
 
+#include "CoinFinite.hpp"
 #include "CoinHelperFunctions.hpp"
 #include "CoinPresolveMatrix.hpp"
 
@@ -57,7 +60,11 @@
  * In the row rep, irow will be eliminated entirely, but not here;
  * icoly is removed from the rows it occurs in.
  */
-static bool elim_tripleton(const char *msg,
+static bool elim_tripleton(const char * 
+#ifdef PRESOLVE_DEBUG
+msg
+#endif
+			   ,
 			   CoinBigIndex *mcstrt, 
 			   double *rlo, double * acts, double *rup,
 			   double *colels,
@@ -179,9 +186,9 @@ static bool elim_tripleton(const char *msg,
 	      return (true);
 	    
 	    // have to adjust various induction variables
-	    kcolx = mcstrt[icolx] + (kcolx - kcsx);
-	    kcsx = mcstrt[icolx];			
-	    kcex = mcstrt[icolx] + hincol[icolx];
+ 	    kcolx = mcstrt[icolx] + (kcolx - kcsx);
+ 	    kcsx = mcstrt[icolx];			
+ 	    kcex = mcstrt[icolx] + hincol[icolx];
 	    kcoly = mcstrt[icoly] + (kcoly - kcs);
 	    kcs = mcstrt[icoly];			// do this for ease of debugging
 	    kce = mcstrt[icoly] + hincol[icoly];
@@ -328,8 +335,9 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
 # endif
   int nactions = 0;
 
-  int *zeros	= new int[ncols];
-  memset(zeros,0,ncols*sizeof(int));
+  int *zeros	= prob->usefulColumnInt_; //new int[ncols];
+  char * mark = reinterpret_cast<char *>(zeros+ncols);
+  memset(mark,0,ncols);
   int nzeros	= 0;
 
   // If rowstat exists then all do
@@ -481,7 +489,7 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
 	  continue;
 	//if (nAdded<=1) 
 	//printf("%d elements added, hincol %d , dups %d\n",nAdded,hincol[icoly],nDuplicate);
-	if (nAdded>1)
+	if (nAdded>2)
 	  continue;
 
 	// it is possible that both x/z and y are singleton columns
@@ -493,6 +501,8 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
 	{
 	  action *s = &actions[nactions];	  
 	  nactions++;
+	  PRESOLVE_DETAIL_PRINT(printf("pre_tripleton %dR %dC %dC %dC E\n",
+				       irow,icoly,icolx,icolz));
 	  
 	  s->row = irow;
 	  s->icolx = icolx;
@@ -597,10 +607,14 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
 	rlo[irow] = 0.0;
 	rup[irow] = 0.0;
 
-	zeros[icolx] = 1;	// check for zeros
-	zeros[icolz] = 1;	// check for zeros
-	nzeros++;
-
+	if (!mark[icolx]) {
+	  mark[icolx]=1;
+	  zeros[nzeros++]=icolx;
+	}
+	if (!mark[icolz]) {
+	  mark[icolz]=1;
+	  zeros[nzeros++]=icolz;
+	}
       }
       
 #     if PRESOLVE_CONSISTENCY
@@ -619,16 +633,11 @@ const CoinPresolveAction *tripleton_action::presolve(CoinPresolveMatrix *prob,
     next = new tripleton_action(nactions, actions1, next);
 
     if (nzeros) {
-      int i;
-      nzeros=0;
-      for (i=0;i<ncols;i++) 
-	if (zeros[i])
-	  zeros[nzeros++]=i;
       next = drop_zero_coefficients_action::presolve(prob, zeros, nzeros, next);
     }
   }
 
-  delete[]zeros;
+  //delete[]zeros;
   deleteAction(actions,action*);
 
   if (prob->tuning_) {
@@ -1047,7 +1056,7 @@ void check_tripletons(const CoinPresolveAction * paction)
     check_tripletons(paction->next);
     
     if (strcmp(paction0->name(), "tripleton_action") == 0) {
-      const tripleton_action *daction = (const tripleton_action *)paction0;
+      const tripleton_action *daction = reinterpret_cast<const tripleton_action *>(paction0);
       for (int i=daction->nactions_-1; i>=0; --i) {
 	int icolx = daction->actions_[i].icolx;
 	int icoly = daction->actions_[i].icoly;
