@@ -1,8 +1,7 @@
 // Bare bones example of using COIN-OR OSI
 
 #include <iostream>
-#include "OsiClpSolverInterface.hpp"
-//#include "OsiGlpkSolverInterface.hpp"
+#include OSIXXXhpp
 
 int
 main(void)
@@ -11,11 +10,15 @@ main(void)
    OsiSolverInterface *si;
 
    // When we instantiate the object, we need a specific derived class.
-   si = new OsiClpSolverInterface;
-   //   si = new OsiGlpkSolverInterface;  
+   si = new OSIXXX;
 
    // Read in an mps file.  This one's from the MIPLIB library.
-   si->readMps("../../Data/Sample/p0033");
+#if defined(SAMPLEDIR)
+   si->readMps(SAMPLEDIR "/p0033");
+#else
+   fprintf(stderr, "Do not know where to find sample MPS files.\n");
+   exit(1);
+#endif
 
    // Solve the (relaxation of the) problem
    si->initialSolve();
