@@ -8,6 +8,7 @@
 // Copyright (C) 2007 Lou Hafer
 //    University of Pittsburgh coding done by Brady Hunsaker
 // All Rights Reserved.
+// This code is licensed under the terms of the Eclipse Public License (EPL).
 //
 // Comments:
 //   
@@ -65,17 +66,18 @@
   repopulate the cache by interrogating glpk.
 */
 
-#if defined(_MSC_VER)
-// Turn off compiler warning about long names
-#  pragma warning(disable:4786)
-#endif
-
 #include <cassert>
+#include <cstdio>
+#include <cmath>
 #include <string>
 #include <iostream>
-#include <stdio.h>
+
+extern "C" {
+#include "glpk.h"
+}
 
 #include "CoinError.hpp"
+#include "CoinPragma.hpp"
 
 #include "OsiConfig.h"
 
@@ -162,6 +164,20 @@ void OGSI::initialSolve()
   Solve the lp.
 */
   int err = lpx_simplex(model) ;
+
+  // for Glpk, a solve fails if the initial basis is invalid or singular
+  // thus, we construct a (advanced) basis first and try again
+#ifdef LPX_E_BADB
+  if (err == LPX_E_BADB) {
+    lpx_adv_basis(model);
+    err = lpx_simplex(model) ;
+  } else
+#endif
+  if (err == LPX_E_SING || err == LPX_E_FAULT) {
+    lpx_adv_basis(model);
+    err = lpx_simplex(model) ;
+  }
+
   iter_used_ = lpx_get_int_parm(model, LPX_K_ITCNT) ;
 /*
   Sort out the various state indications.
@@ -201,6 +217,9 @@ void OGSI::initialSolve()
     } // no break here, so we still report abandoned
     case LPX_E_FAULT:
     case LPX_E_SING:
+#ifdef LPX_E_BADB
+    case LPX_E_BADB:
+#endif
     { isAbandoned_ = true ;
       break ; }
     case LPX_E_NOPFS:
@@ -242,14 +261,18 @@ void OGSI::resolve()
   // lpx_simplex will use the current basis if possible
   int err = lpx_simplex(model) ;
 
-#ifdef LPX_E_BADB
-  // in Glpk 4.30 a resolve fails if the initial basis is invalid
+  // for Glpk, a solve fails if the initial basis is invalid or singular
   // thus, we construct a (advanced) basis first and try again
+#ifdef LPX_E_BADB
   if (err == LPX_E_BADB) {
-  	lpx_adv_basis(model);
-  	err = lpx_simplex(model) ;
-  }
+    lpx_adv_basis(model);
+    err = lpx_simplex(model) ;
+  } else
 #endif
+  if (err == LPX_E_SING || err == LPX_E_FAULT) {
+    lpx_adv_basis(model);
+    err = lpx_simplex(model) ;
+  }
   
   iter_used_ = lpx_get_int_parm(model,LPX_K_ITCNT) ;
 
@@ -279,6 +302,9 @@ void OGSI::resolve()
     } // no break here, so we still report abandoned
     case LPX_E_FAULT:
     case LPX_E_SING:
+#ifdef LPX_E_BADB
+    case LPX_E_BADB:
+#endif
     { isAbandoned_ = true ;
       break ; }
     case LPX_E_NOPFS:
@@ -379,6 +405,9 @@ void OGSI::branchAndBound ()
 
     switch (err)
     { case LPX_E_OK:
+#ifdef LPX_E_MIPGAP
+    case LPX_E_MIPGAP:
+#endif
       { 
 	break ; }
       case LPX_E_NOPFS:
@@ -549,6 +578,8 @@ bool OGSI::setStrParam (OsiStrParam key,
   switch (key)
   { case OsiProbName:
     { probName_ = value ;
+      if (probName_.length() == 0)
+        probName_ = "Pb";
       lpx_set_prob_name(lp_,const_cast<char *>(value.c_str())) ;
       retval = true ;
       break ; }
@@ -1730,7 +1761,7 @@ int OGSI::getIterationCount() const
 
 //-----------------------------------------------------------------------------
 
-std::vector<double*> OGSI::getDualRays(int maxNumRays) const
+std::vector<double*> OGSI::getDualRays(int /*maxNumRays*/, bool /*fullRay*/) const
 {
 	// ??? not yet implemented.
 	throw CoinError("method is not yet implemented", "getDualRays", "OsiGlpkSolverInterface");
@@ -1739,7 +1770,7 @@ std::vector<double*> OGSI::getDualRays(int maxNumRays) const
 
 //-----------------------------------------------------------------------------
 
-std::vector<double*> OGSI::getPrimalRays(int maxNumRays) const
+std::vector<double*> OGSI::getPrimalRays(int /*maxNumRays*/) const
 {
 	// ??? not yet implemented.
 	throw CoinError("method is not yet implemented", "getPrimalRays", "OsiGlpkSolverInterface");
@@ -2746,12 +2777,10 @@ OGSI::loadProblem(const int numcols, const int numrows,
 				   const char* rowsen, const double* rowrhs,   
 				   const double* rowrng)
 {
-   assert( rowsen != NULL );
-   assert( rowrhs != NULL );
    double * rowlb = new double[numrows];
    double * rowub = new double[numrows];
    for (int i = numrows-1; i >= 0; --i) {   
-      convertSenseToBound(rowsen[i],rowrhs[i],rowrng[i],rowlb[i],rowub[i]);
+      convertSenseToBound(rowsen != NULL ? rowsen[i] : 'G', rowrhs != NULL ? rowrhs[i] : 0.0, rowrng != NULL ? rowrng[i] : 0.0, rowlb[i], rowub[i]);
    }
 
    loadProblem( numcols, numrows, start, index, value, collb, colub, obj,
@@ -2784,7 +2813,7 @@ int OGSI::readMps (const char *filename,
 
 void OGSI::writeMps( const char * filename,
 				       const char * extension,
-				       double objSense ) const
+		     double /*objSense*/ ) const
 {
 	// Could be in OsiSolverInterfaceImpl.
 #if 1
@@ -3140,7 +3169,9 @@ void OGSI::gutsOfCopy (const OsiGlpkSolverInterface &source)
     { int stati = lpx_get_row_stat(srclpx,i) ;
       lpx_set_row_stat(lpx,i,stati) ; }
 
+#ifndef NDEBUG
     int retval = lpx_warm_up(lpx) ;
+#endif
 #   if OGSI_TRACK_SOLVERS > 1
     std::cout
       << "    lpx_warm_up returns " << retval << "." << std::endl ;
@@ -3219,9 +3250,13 @@ void OGSI::gutsOfConstructor()
 
   lpx_set_prob_name(lp_,const_cast<char *>(probName_.c_str())) ;
 
-  lpx_set_int_parm(lp_,LPX_K_PRESOL,0) ;
-  lpx_set_int_parm(lp_,LPX_K_DUAL,1) ;
-  lpx_set_int_parm(lp_,LPX_K_SCALE,3) ;
+/* Stefan: with the new simplex algorithm in Glpk 4.31, some netlib instances (e.g., dfl001)
+   take very long or get into a cycle.
+   Thus, let's try to leave parameters onto their defaults.
+*/
+//  lpx_set_int_parm(lp_,LPX_K_PRESOL,0) ;
+//  lpx_set_int_parm(lp_,LPX_K_DUAL,1) ;
+//  lpx_set_int_parm(lp_,LPX_K_SCALE,3) ;
 /*
   Printing is a bit more complicated. Set the log level in the handler and set
   the print parameter in glpk.
