@@ -1,4 +1,4 @@
-#include <malloc.h>
+#include <memory.h>
 #include <string.h>
 
 #include "decomp.h"
