@@ -2,12 +2,12 @@
 /*                                                                           */
 /* This file is part of the SYMPHONY MILP Solver Framework.                  */
 /*                                                                           */
-/* SYMPHONY was jointly developed by Ted Ralphs (tkralphs@lehigh.edu) and    */
+/* SYMPHONY was jointly developed by Ted Ralphs (ted@lehigh.edu) and         */
 /* Laci Ladanyi (ladanyi@us.ibm.com).                                        */
 /*                                                                           */
-/* (c) Copyright 2000-2008 Ted Ralphs. All Rights Reserved.                  */
+/* (c) Copyright 2000-2011 Ted Ralphs. All Rights Reserved.                  */
 /*                                                                           */
-/* This software is licensed under the Common Public License. Please see     */
+/* This software is licensed under the Eclipse Public License. Please see    */
 /* accompanying file for terms.                                              */
 /*                                                                           */
 /*===========================================================================*/
@@ -51,11 +51,14 @@ int main(int argc, char **argv)
 #include "symphony.h"
 #include "sym_master.h"
 #include "sym_messages.h"
-#ifdef HAS_READLINE
+#if defined HAS_READLINE
 #include <pwd.h>
 #include <readline/readline.h>
 #include <readline/history.h>
 
+#if (RL_VERSION_MAJOR == 5 && RL_VERSION_MINOR >= 2) || \
+   (RL_VERSION_MAJOR >= 6)
+
 typedef struct {
   char *name;			
 } COMMAND;
@@ -125,6 +128,8 @@ void sym_initialize_readline();
 char *command_generator (const char *text, int state);
 char *alloc_str (char *s);
 void sym_read_tilde(char input[]);
+
+#endif
 #endif
 
 int comp_level = 0;
@@ -150,7 +155,7 @@ int main(int argc, char **argv)
       sym_parse_command_line(env, argc, argv);
 
       if (env->par.verbosity >= 0){
-	 version();
+	 sym_version();
       }
       
       if (env->par.test){
@@ -172,7 +177,7 @@ int main(int argc, char **argv)
 	    printf("See sym_return_values.h for meaning of code.\n\n");
 	    exit(termcode);
 	 }
-
+	 printf("\n");
 	 sym_solve(env);
       }
    
@@ -189,13 +194,14 @@ int main(int argc, char **argv)
      double objval = 0.0, initial_time = 0.0, start_time = 0.0;
      double finish_time = 0.0, dbl_value = 0;
 
-     version();
-     printf("***** WELCOME TO SYMPHONY INTERACTIVE MIP SOLVER ******\n\n"
-	    "Please type 'help'/'?' to see the main commands!\n\n");
+     sym_version();
+     printf("Please type 'help'/'?' to see a list of commands\n\n");
 
-     env->par.verbosity = -1;
+     sym_set_int_param(env, "verbosity", -1);
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+ 			      (RL_VERSION_MAJOR >= 6))
      sym_initialize_readline();
 #endif
      
@@ -219,7 +225,9 @@ int main(int argc, char **argv)
 	   strcpy(args[1], line);
 	 }	 
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
 	 sym_read_tilde(args[1]);	 
 #endif	 	 
 	 if (fopen(args[1], "r") == NULL){
@@ -276,7 +284,9 @@ int main(int argc, char **argv)
 	     strcpy(args[2], line);
 	   }
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
 	   sym_read_tilde(args[2]);	 
 #endif	 	 
 	 
@@ -299,12 +309,14 @@ int main(int argc, char **argv)
 	 } 
 	 if(strcmp(args[0], "solve") == 0){
 	   start_time = wall_clock(NULL);
+	   printf("\n");
 	   termcode = sym_solve(env);
 	   finish_time = wall_clock(NULL);
 	 } else {
 	   is_int = env->mip->is_int;
 	   env->mip->is_int  = (char *)   calloc(CSIZE, env->mip->n);
 	   start_time = wall_clock(NULL);
+	   printf("\n");
 	   termcode = sym_solve(env);
 	   finish_time = wall_clock(NULL);
 	   env->mip->is_int = is_int;
@@ -424,10 +436,11 @@ int main(int argc, char **argv)
 	       
 	       print_statistics(&(env->warm_start->comp_times), 
 				&(env->warm_start->stat),
+                                NULL,
 				env->warm_start->ub, env->warm_start->lb, 
 				initial_time, start_time, finish_time,
 				env->mip->obj_offset, env->mip->obj_sense,
-				env->warm_start->has_ub);
+				env->warm_start->has_ub,NULL);
 	       printf("\n");	       
 	     }
 	     strcpy(args[1], "");	       
@@ -512,7 +525,9 @@ int main(int argc, char **argv)
 	       strcpy(args[2], line);
 	     }
 
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) ||\
+			      (RL_VERSION_MAJOR >= 6))
 	     sym_read_tilde(args[2]);	 
 #endif	 	 
 
@@ -567,7 +582,7 @@ int main(int argc, char **argv)
 	 printf("Resetting...\n");
 	 sym_close_environment(env);
 	 env = sym_open_environment();
-	 env->par.verbosity = -1;
+	 sym_set_int_param(env, "verbosity", -1);
        } else if ((strcmp(args[0], "quit") == 0) ||
 		  (strcmp(args[0], "exit") == 0)){
 	 break;
@@ -606,7 +621,7 @@ int sym_help(const char *line)
   } else if (strcmp(line, "set_help") == 0 || strcmp(line, "display_param_help") == 0){
 
     printf("\n\nList of parameters: \n\n"); 
-    printf("verbosity                          : set verbosity (default: 1)\n"
+    printf("verbosity                          : set verbosity (default: -1)\n"
 	   "upper_bound                        : use an initial upper bound\n"
 	   "find_first_feasible                : whether to find the first feasible solution or\n"
 	   "                                     to solve the optimality (default: 0) \n"
@@ -661,7 +676,25 @@ int sym_help(const char *line)
 int sym_read_line(const char *prompt, char **input)
 {
 
-#ifndef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
+
+  if (*input) FREE(*input);
+
+  while(true) {
+    *input = readline(prompt);
+    if (**input) {      
+      add_history(*input);
+      if((*input)[strlen(*input)-1] == ' '){
+	(*input)[strlen(*input)-1] = 0;
+      }
+      break;
+    } else continue;
+  }
+
+#else
+  
   int i;
 
   if (*input) FREE(*input);
@@ -669,7 +702,7 @@ int sym_read_line(const char *prompt, char **input)
    
   while(true){
      strcpy(getl, "");
-     printf(prompt);
+     printf("%s",prompt);
      fflush(stdout);
      fgets(getl, MAX_LINE_LENGTH, stdin);
 
@@ -687,23 +720,8 @@ int sym_read_line(const char *prompt, char **input)
   }
   *input = getl;
   
-#else
-
-  if (*input) FREE(*input);
-
-  while(true) {
-    *input = readline(prompt);
-    if (**input) {      
-      add_history(*input);
-      if((*input)[strlen(*input)-1] == ' '){
-	(*input)[strlen(*input)-1] = 0;
-      }
-      break;
-    } else continue;
-  }
-
 #endif
-  
+
   return (0);
 }
 
@@ -724,7 +742,9 @@ int sym_free_env(sym_environment *env){
 } 
 /*===========================================================================*\
 \*===========================================================================*/
-#ifdef HAS_READLINE
+#if defined(HAS_READLINE) && ((RL_VERSION_MAJOR == 5 && \
+			       RL_VERSION_MINOR >= 2) || \
+			      (RL_VERSION_MAJOR >= 6))
 
 void sym_initialize_readline()
 {
